/*
 *       @(#) xtcio.c 1.9 12 Nov 1995
 *
 *       This source code is part of
 *
 *        G   R   O   M   A   C   S
 *
 * GROningen MAchine for Chemical Simulations
 *
 * Copyright (c) 1990-1995,
 * BIOSON Research Institute, Dept. of Biophysical Chemistry,
 * University of Groningen, The Netherlands
 *
 * Please refer to:
 * GROMACS: A message-passing parallel molecular dynamics implementation
 * H.J.C. Berendsen, D. van der Spoel and R. van Drunen
 * Comp. Phys. Comm. 91, 43-56 (1995)
 *
 * Also check out our WWW page:
 * http://rugmd0.chem.rug.nl/~gmx
 * or e-mail to:
 * gromacs@chem.rug.nl
 *
 * And Hey:
 * Good ROcking Metal Altar for Chronical Sinners
 */
#include <string.h>
#include "typedefs.h"
#include "xdrf.h"
#include "xtcio.h"
#include "smalloc.h"
#include "vec.h"
#include "futil.h"
#include "fatal.h"

#define XTC_MAGIC 1995

/* 
 * Open XTC file 
 */
int open_xtc(XDR *xd,char *filename,char *mode)
{
  char *bf;
  
  if ((strcmp(mode,"w")==0)&&fexist(filename)) {
    bf=(char *)backup_fn(filename);
    if (rename(filename,bf) == 0) {
      fprintf(stderr,"\nBack Off! I just backed up %s to %s\n",filename,bf);
    }
    else
      fprintf(stderr,"Sorry, I couldn't backup %s to %s\n",filename,bf);
  }
  if ( !xdropen(xd,filename,mode) ) {
    perror(filename);
    exit(1);
  }
  return 0;
}

/*
 * Close XTC file
 */
void close_xtc(XDR *xd)
{
  xdrclose(xd);
}

static void check_xtc_magic(int magic)
{
  if (magic != XTC_MAGIC) 
    fatal_error(0,"Magic Number Error in XTC file (read %d, should be %d)",
		magic,XTC_MAGIC);
}

void xtc_check(char *str,bool bResult,char *file,int line)
{
  if (!bResult) {
    fprintf(stderr,"XTC error: read/write of %s failed, "
	    "source file %s, line %d\n",str,file,line);
    exit(1);
  }
}

static int xtc_header(XDR *xd,int *magic,int *natoms,int *step,real *time)
{
  if (xdr_int(xd,magic) == 0)
    return 0;
  XTC_CHECK("natoms", xdr_int(xd,natoms));  /* number of atoms */
  XTC_CHECK("step",   xdr_int(xd,step));    /* frame number    */
  XTC_CHECK("time",   xdr_real(xd,time));   /* time            */
    
  return 1;
}

static int xtc_coord(XDR *xd,
		     int *natoms,
		     matrix box,rvec *x,real *prec)
{
  int i,j;
  
  /* box */
  for(i=0; (i<DIM); i++)
    for(j=0; (j<DIM); j++)
      XTC_CHECK("box",xdr_real(xd,&(box[i][j])));
  
  /* coordinates     */
  XTC_CHECK("x",xdr3drcoord(xd,x[0],natoms,prec)); 
  
  return 1;
}

static int xtc_io(XDR *xd,int *magic,
		  int *natoms,int *step,real *time,
		  matrix box,rvec *x,real *prec)
{
  if (!xtc_header(xd,magic,natoms,step,time))
    return 0;
  return xtc_coord(xd,natoms,box,x,prec);
}

int write_xtc(XDR *xd,
	      int natoms,int step,real time,
	      matrix box,rvec *x,real prec)
{
  int magic_number = XTC_MAGIC;
  
  /* write magic number and xtc identidier */
  if (!xtc_header(xd,&magic_number,&natoms,&step,&time))
    return 0;
    
  /* write data */
  return xtc_coord(xd,&natoms,box,x,&prec);
}

int read_first_xtc(XDR *xd,char *filename,
		   int *natoms,int *step,real *time,
		   matrix box,rvec **x,real *prec)
{
  int magic;

  /* Open the ecstacy XTC file */
  open_xtc(xd,filename,"r");
  
  /* read header and malloc x */
  if ( !xtc_header(xd,&magic,natoms,step,time))
    return 0;
    
  /* Check magic number */
  check_xtc_magic(magic);
  
  snew(*x,*natoms);
  
  return xtc_coord(xd,natoms,box,*x,prec);
}

int read_next_xtc(XDR *xd,
		  int *natoms,int *step,real *time,
		  matrix box,rvec *x,real *prec)
{
  int magic;

  /* read header */
  if ( !xtc_header(xd,&magic,natoms,step,time))
    return 0;
    
  /* Check magic number */
  check_xtc_magic(magic);
  
  return xtc_coord(xd,natoms,box,x,prec);
}

